package betterquesting.network.handlers;

import betterquesting.api.api.QuestingAPI;
import betterquesting.api.network.QuestingPacket;
import betterquesting.api.questing.IQuest;
import betterquesting.api.questing.IQuestDatabase;
import betterquesting.api.questing.IQuestLine;
import betterquesting.api.questing.IQuestLineDatabase;
import betterquesting.api.questing.IQuestLineEntry;
import betterquesting.api2.storage.DBEntry;
import betterquesting.api2.utils.Tuple2;
import betterquesting.client.importers.ImportedQuestLines;
import betterquesting.client.importers.ImportedQuests;
import betterquesting.core.BetterQuesting;
import betterquesting.handlers.SaveLoadHandler;
import betterquesting.network.PacketSender;
import betterquesting.network.PacketTypeRegistry;
import betterquesting.questing.QuestDatabase;
import betterquesting.questing.QuestLineDatabase;
import com.mojang.realmsclient.gui.ChatFormatting;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import javax.annotation.Nonnull;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ResourceLocation;
import org.apache.logging.log4j.Level;

/* loaded from: betterquesting_network_handlers.jar:betterquesting/network/handlers/NetImport.class */
public class NetImport {
    private static final ResourceLocation ID_NAME = new ResourceLocation("betterquesting:import");

    public static void registerHandler() {
        PacketTypeRegistry.INSTANCE.registerServerHandler(ID_NAME, NetImport::onServer);
    }

    public static void sendImport(@Nonnull IQuestDatabase questDB, @Nonnull IQuestLineDatabase chapterDB) {
        NBTTagCompound payload = new NBTTagCompound();
        payload.func_74782_a("quests", questDB.writeToNBT(new NBTTagList(), (List) null));
        payload.func_74782_a("chapters", chapterDB.writeToNBT(new NBTTagList(), (List) null));
        PacketSender.INSTANCE.sendToServer(new QuestingPacket(ID_NAME, payload));
    }

    private static void onServer(Tuple2<NBTTagCompound, EntityPlayerMP> message) {
        EntityPlayerMP sender = (EntityPlayerMP) message.func_76340_b();
        if (sender.field_71133_b == null) {
            return;
        }
        boolean isOP = sender.field_71133_b.func_71203_ab().func_152596_g(sender.func_146103_bH());
        if (!isOP) {
            BetterQuesting.logger.log(Level.WARN, "Player " + sender.func_70005_c_() + " (UUID:" + QuestingAPI.getQuestingUUID(sender) + ") tried to import quests without OP permissions!");
            sender.func_146105_b(new ChatComponentText(ChatFormatting.RED + "You need to be OP to edit quests!"));
            return;
        }
        ImportedQuests impQuestDB = new ImportedQuests();
        ImportedQuestLines importedQuestLines = new ImportedQuestLines();
        impQuestDB.readFromNBT(((NBTTagCompound) message.func_76341_a()).func_150295_c("quests", 10), false);
        importedQuestLines.readFromNBT(((NBTTagCompound) message.func_76341_a()).func_150295_c("chapters", 10), false);
        BetterQuesting.logger.log(Level.INFO, "Importing " + impQuestDB.size() + " quest(s) and " + importedQuestLines.size() + " quest line(s) from " + sender.func_146103_bH().getName());
        HashMap<Integer, Integer> remapped = getRemappedIDs(impQuestDB.getEntries());
        for (DBEntry<IQuest> entry : impQuestDB.getEntries()) {
            int[] oldIDs = Arrays.copyOf(((IQuest) entry.getValue()).getRequirements(), ((IQuest) entry.getValue()).getRequirements().length);
            for (int n = 0; n < oldIDs.length; n++) {
                if (remapped.containsKey(Integer.valueOf(oldIDs[n]))) {
                    oldIDs[n] = remapped.get(Integer.valueOf(oldIDs[n])).intValue();
                }
            }
            ((IQuest) entry.getValue()).setRequirements(oldIDs);
            QuestDatabase.INSTANCE.add(remapped.get(Integer.valueOf(entry.getID())).intValue(), entry.getValue());
        }
        for (DBEntry<IQuestLine> questLine : importedQuestLines.getEntries()) {
            List<DBEntry<IQuestLineEntry>> pendingQLE = new ArrayList<>();
            for (DBEntry<IQuestLineEntry> qle : ((IQuestLine) questLine.getValue()).getEntries()) {
                pendingQLE.add(qle);
                ((IQuestLine) questLine.getValue()).removeID(qle.getID());
            }
            for (DBEntry<IQuestLineEntry> qle2 : pendingQLE) {
                if (!remapped.containsKey(Integer.valueOf(qle2.getID()))) {
                    BetterQuesting.logger.error("Failed to import quest into quest line. Unable to remap ID " + qle2.getID());
                } else {
                    ((IQuestLine) questLine.getValue()).add(remapped.get(Integer.valueOf(qle2.getID())).intValue(), qle2.getValue());
                }
            }
            QuestLineDatabase.INSTANCE.add(QuestLineDatabase.INSTANCE.nextID(), questLine.getValue());
        }
        SaveLoadHandler.INSTANCE.markDirty();
        NetQuestSync.quickSync(-1, true, true);
        NetChapterSync.sendSync(null, null);
    }

    private static HashMap<Integer, Integer> getRemappedIDs(List<DBEntry<IQuest>> idList) {
        int[] nextIDs = getNextIDs(idList.size());
        HashMap<Integer, Integer> remapped = new HashMap<>();
        for (int i = 0; i < nextIDs.length; i++) {
            remapped.put(Integer.valueOf(idList.get(i).getID()), Integer.valueOf(nextIDs[i]));
        }
        return remapped;
    }

    private static int[] getNextIDs(int num) {
        List<DBEntry<IQuest>> listDB = QuestDatabase.INSTANCE.getEntries();
        int[] nxtIDs = new int[num];
        if (listDB.size() <= 0 || listDB.get(listDB.size() - 1).getID() == listDB.size() - 1) {
            for (int i = 0; i < num; i++) {
                nxtIDs[i] = listDB.size() + i;
            }
            return nxtIDs;
        }
        int n1 = 0;
        int n2 = 0;
        for (int i2 = 0; i2 < num; i2++) {
            while (n2 < listDB.size() && listDB.get(n2).getID() == n1) {
                n1++;
                n2++;
            }
            int i3 = n1;
            n1++;
            nxtIDs[i2] = i3;
        }
        return nxtIDs;
    }
}
